class ItemList:
  items = [
    {
      "id": 1,
      "kind": "weapon",
      "name": "Club",
      "level": 1,
      "ac": "",
      "atk": 1,
      "type": "blunt",
      "ability": "",
      "effect": ""
    },
    {
      "id": 2,
      "kind": "weapon",
      "name": "Hatchet",
      "level": 1,
      "ac": "",
      "atk": 1,
      "type": "slash",
      "ability": "",
      "effect": ""
    },
    {
      "id": 3,
      "kind": "weapon",
      "name": "Knife",
      "level": 1,
      "ac": "",
      "atk": 1,
      "type": "pierce",
      "ability": "",
      "effect": ""
    },
    {
      "id": 4,
      "kind": "weapon",
      "name": "Axe",
      "level": 2,
      "ac": "",
      "atk": 2,
      "type": "slash",
      "ability": "",
      "effect": ""
    },
    {
      "id": 5,
      "kind": "weapon",
      "name": "Staff",
      "level": 2,
      "ac": "",
      "atk": 2,
      "type": "blunt",
      "ability": "",
      "effect": ""
    },
    {
      "id": 6,
      "kind": "weapon",
      "name": "Dagger",
      "level": 2,
      "ac": "",
      "atk": 2,
      "type": "pierce",
      "ability": "",
      "effect": ""
    },
    {
      "id": 7,
      "kind": "weapon",
      "name": "Sword",
      "level": 3,
      "ac": "",
      "atk": 3,
      "type": "slash",
      "ability": "",
      "effect": ""
    },
    {
      "id": 8,
      "kind": "weapon",
      "name": "Mace",
      "level": 3,
      "ac": "",
      "atk": 3,
      "type": "blunt",
      "ability": "",
      "effect": ""
    },
    {
      "id": 9,
      "kind": "weapon",
      "name": "Spear",
      "level": 3,
      "ac": "",
      "atk": 3,
      "type": "pierce",
      "ability": "",
      "effect": ""
    },
    {
      "id": 10,
      "kind": "weapon",
      "name": "Claymore",
      "level": 4,
      "ac": "",
      "atk": 4,
      "type": "slash",
      "ability": "",
      "effect": ""
    },
    {
      "id": 11,
      "kind": "weapon",
      "name": "Warhammer",
      "level": 4,
      "ac": "",
      "atk": 4,
      "type": "blunt",
      "ability": "",
      "effect": ""
    },
    {
      "id": 12,
      "kind": "weapon",
      "name": "Rapier",
      "level": 4,
      "ac": "",
      "atk": 4,
      "type": "pierce",
      "ability": "",
      "effect": ""
    },
    {
      "id": 13,
      "kind": "weapon",
      "name": "Flamberge",
      "level": 5,
      "ac": "",
      "atk": 5,
      "type": "slash",
      "ability": "",
      "effect": ""
    },
    {
      "id": 14,
      "kind": "weapon",
      "name": "Morningstar",
      "level": 5,
      "ac": "",
      "atk": 5,
      "type": "blunt",
      "ability": "",
      "effect": ""
    },
    {
      "id": 15,
      "kind": "weapon",
      "name": "Pick",
      "level": 5,
      "ac": "",
      "atk": 5,
      "type": "pierce",
      "ability": "",
      "effect": ""
    },
    {
      "id": 16,
      "kind": "weapon",
      "name": "Glaive",
      "level": 6,
      "ac": "",
      "atk": 6,
      "type": "slash",
      "ability": "",
      "effect": ""
    },
    {
      "id": 17,
      "kind": "weapon",
      "name": "Flail",
      "level": 6,
      "ac": "",
      "atk": 6,
      "type": "blunt",
      "ability": "",
      "effect": ""
    },
    {
      "id": 18,
      "kind": "weapon",
      "name": "Pike",
      "level": 6,
      "ac": "",
      "atk": 6,
      "type": "pierce",
      "ability": "",
      "effect": ""
    },
    {
      "id": 19,
      "kind": "weapon",
      "name": "Halberd",
      "level": 7,
      "ac": "",
      "atk": 7,
      "type": "slash",
      "ability": "",
      "effect": ""
    },
    {
      "id": 20,
      "kind": "weapon",
      "name": "Maul",
      "level": 7,
      "ac": "",
      "atk": 7,
      "type": "blunt",
      "ability": "",
      "effect": ""
    },
    {
      "id": 21,
      "kind": "weapon",
      "name": "Trident",
      "level": 7,
      "ac": "",
      "atk": 7,
      "type": "pierce",
      "ability": "",
      "effect": ""
    },
    {
      "id": 22,
      "kind": "armor",
      "name": "Robe",
      "level": 1,
      "ac": 1,
      "atk": "",
      "type": "cloth",
      "ability": "",
      "effect": ""
    },
    {
      "id": 23,
      "kind": "armor",
      "name": "Cuirboulli",
      "level": 2,
      "ac": 2,
      "atk": "",
      "type": "leather",
      "ability": "",
      "effect": ""
    },
    {
      "id": 24,
      "kind": "armor",
      "name": "Hauberk",
      "level": 3,
      "ac": 3,
      "atk": "",
      "type": "metal",
      "ability": "",
      "effect": ""
    },
    {
      "id": 25,
      "kind": "armor",
      "name": "Jack",
      "level": 4,
      "ac": 4,
      "atk": "",
      "type": "leather",
      "ability": "",
      "effect": ""
    },
    {
      "id": 26,
      "kind": "armor",
      "name": "Brigandine",
      "level": 5,
      "ac": 5,
      "atk": "",
      "type": "metal",
      "ability": "",
      "effect": ""
    },
    {
      "id": 27,
      "kind": "armor",
      "name": "Breastplate",
      "level": 6,
      "ac": 6,
      "atk": "",
      "type": "metal",
      "ability": "",
      "effect": ""
    },
    {
      "id": 28,
      "kind": "armor",
      "name": "Cuirass",
      "level": 7,
      "ac": 7,
      "atk": "",
      "type": "metal",
      "ability": "",
      "effect": ""
    },
    {
      "id": 29,
      "kind": "ring",
      "name": "Ring of Shielding",
      "level": 3,
      "ac": 1,
      "atk": "",
      "type": "",
      "ability": "",
      "effect": ""
    },
    {
      "id": 30,
      "kind": "ring",
      "name": "Ring",
      "level": 2,
      "ac": "",
      "atk": "",
      "type": "",
      "ability": "traveling",
      "effect": ""
    },
    {
      "id": 31,
      "kind": "ring",
      "name": "Ring",
      "level": 3,
      "ac": "",
      "atk": "",
      "type": "",
      "ability": "running",
      "effect": ""
    },
    {
      "id": 32,
      "kind": "ring",
      "name": "Ring",
      "level": 4,
      "ac": "",
      "atk": "",
      "type": "",
      "ability": "regeneration",
      "effect": ""
    },
    {
      "id": 33,
      "kind": "usable",
      "name": "Potion",
      "level": 1,
      "ac": "",
      "atk": "",
      "type": "anytime",
      "ability": "",
      "effect": "healing"
    },
    {
      "id": 34,
      "kind": "usable",
      "name": "Scroll",
      "level": 1,
      "ac": "",
      "atk": "",
      "type": "peace",
      "ability": "",
      "effect": "digging"
    },
    {
      "id": 35,
      "kind": "usable",
      "name": "Scroll",
      "level": 1,
      "ac": "",
      "atk": "",
      "type": "anytime",
      "ability": "",
      "effect": "blinking"
    },
    {
      "id": 36,
      "kind": "usable",
      "name": "Scroll",
      "level": 2,
      "ac": "",
      "atk": "",
      "type": "peace",
      "ability": "",
      "effect": "teleport"
    },
    {
      "id": 37,
      "kind": "usable",
      "name": "Potion",
      "level": 2,
      "ac": "",
      "atk": "",
      "type": "combat",
      "ability": "",
      "effect": "invisibility"
    },
    {
      "id": 38,
      "kind": "usable",
      "name": "Scroll",
      "level": 2,
      "ac": "",
      "atk": "",
      "type": "peace",
      "ability": "",
      "effect": "shopping"
    },
    {
      "id": 39,
      "kind": "usable",
      "name": "Scroll",
      "level": 4,
      "ac": "",
      "atk": "",
      "type": "peace",
      "ability": "",
      "effect": "time warp"
    },
    {
      "id": 40,
      "kind": "usable",
      "name": "Scroll",
      "level": 3,
      "ac": "",
      "atk": "",
      "type": "combat",
      "ability": "",
      "effect": "fireball"
    },
    {
      "id": 41,
      "kind": "usable",
      "name": "Potion",
      "level": 4,
      "ac": "",
      "atk": "",
      "type": "peace",
      "ability": "",
      "effect": "knowledge"
    }
  ]

  descriptors = {
      "weapon": {
          "pierce": [
              "Light",
              "Balanced",
              "Tapered",
              "Pointy",
              "Sharp",
              "Keen",
              "Piercing",
              "Barbed",
              "Impaling",
              "Stabbing",
              "Needle",
              "Diamond"
          ],
          "slash": [
              "Fine",
              "Long",
              "Honed",
              "Jagged",
              "Razor",
              "Gleaming",
              "Serrated",
              "Tempered",
              "Curved",
              "Keen",
              "Double Edged",
              "Vorpal"
          ],
          "blunt": [
              "Polished",
              "Hardened",
              "Dense",
              "Faceted",
              "Iron",
              "Heavy",
              "Leaden",
              "Massive",
              "Dragonbone",
              "Crushing",
              "Giant",
              "Unstoppable"
          ]
      },
      "armor": {
          "cloth": [
              "Linen",
              "Thick",
              "Wool",
              "Striped",
              "Felted",
              "Patterned",
              "Silk",
              "Embroidered",
              "Brocade",
              "Velvet",
              "Goldworked",
              "Mageweave"
          ],
          "leather": [
              "Soft",
              "Deerskin",
              "Waxed",
              "Hardened",
              "Dyed",
              "Embossed",
              "Tooled",
              "Lacquered",
              "Rhino Hide",
              "Gilded",
              "Trollskin",
              "Dragonscale"
          ],
          "metal": [
              "Copper",
              "Bronze",
              "Shiny",
              "Iron",
              "Polished",
              "Steel",
              "Burnished",
              "Exotic",
              "Engraved",
              "Chased",
              "Mithril",
              "Adamantine"
          ]
      },
      "ring": [
          "Shiny",
          "Golden",
          "Amethyst",
          "Agate",
          "Lustrous",
          "Garnet",
          "Malachite",
          "Hematite",
          "Emerald",
          "Elvish",
          "Ruby",
          "Dwarvish",
          "Sapphire",
          "Starmetal",
          "Glowing",
          "Quicksilver",
          "Crystal",
          "Dragonbone",
        
      ]
  }

  atkVerb = {
      "slash": [
          "slash",
          "cut",
          "hack"
      ],
      "pierce": [
          "stab",
          "poke",
          "impale"
      ],
      "blunt": [
          "bash",
          "smash",
          "crush"
      ],
      "acid": [
          "dissolve",
          "burn",
          "melt"
      ],
      "fire": [
          "scorch",
          "burn",
          "char"
      ],
      "electric": [
          "shock",
          "zap",
          "jolt"
      ],
      "cold": [
          "freeze",
          "chill",
          "frost"
      ]
  }

  weaponEgo = [
    {
      "name": "Flame",
      "bonus": 1,
      "attributes": {
        "type": "fire"
      }
    },
    {
      "name": "Frost",
      "bonus": 1,
      "attributes": {
        "type": "cold"
      }
    },
    {
      "name": "Acid",
      "bonus": 1,
      "attributes": {
        "type": "acid"
      }
    },
    {
      "name": "Shock",
      "bonus": 1,
      "attributes": {
        "type": "electric"
      }
    },
    {
      "name": "Power",
      "bonus": 2,
      "attributes": {}
    }
  ]

  armorEgo = [
    {
      "name": "Cooling",
      "bonus": 1,
      "attributes": {
        "ability": "resist_fire"
      }
    },
    {
      "name": "Warmth",
      "bonus": 1,
      "attributes": {
        "ability": "resist_cold"
      }
    },
    {
      "name": "Alkali",
      "bonus": 1,
      "attributes": {
        "ability": "resist_acid"
      }
    },
    {
      "name": "Insulation",
      "bonus": 1,
      "attributes": {
        "ability": "resist_electric"
      }
    },
    {
      "name": "Health",
      "bonus": 0,
      "attributes": {
        "ability": "regeneration" 
      }
    },
    {
      "name": "Speed",
      "bonus": 0,
      "attributes": {
        "ability": "running" 
      }
    },
    {
      "name": "Defense",
      "bonus": 2,
      "attributes": {}
    }
  ]